import numpy as np
import pylab as pl
from precision_matrix import clean_signals
from scipy import io, linalg
import itertools as it
from ips import prop_scaling
from sklearn.covariance import LedoitWolf
from StrucSup import StrucSup 
from my_cs_amd import cs_amdW
from run_randomisedLasso import run_randomisedLasso
from prepChol import prepChol
from visual_res import visual_res
import pdb
import random as rand
import os

save_flag = 1

#percentage of connections to include
perCent = 60./100.
perCentS = '60_100'
methodN = 'amd'       #'amd', 'rand'-> create a random vector, 'symamd'->load ordering from text (produced by matlab symamd)


#subject ids
folder_names00 = [ '3188','3227','3295','3323','3333','3367','3371','3382','3388N','3397N','3409','3415','3422','3455','4504','4523','4524','4525','4526','4546','4563','4662','5185','5313','5587','5599','5020','5031','5067','5110','5142','5156','5193','5217','5301','5735','5821','5934','6125','6152','6160' ]
#folder_names = [ '3188','3227','3295' ]
subjNum = len(folder_names00)

#declare which functional connections to examine
#roisF = [2,0]
#thres = 0.2;

#vector with indeces to the rois selected
#rois = range(0,54)
rois = range(4,16)+range(21,33)+range(49,73)+range(75,77)+range(79,83)
#rois = range(0,18)
lenR = len(rois)
prefS=str(lenR)

#output main directory: intermediate results and final are written there
#pathMainDir='../preprocessing/INRIA/subj41_RSNsG'
pathMainDir = '../preprocessing/INRIA/subj41_EBI'

#directory for output of bootstrap results
pathBootIn = pathMainDir + '/' + 'bootstrap_res/'
pathBootOut = pathBootIn + perCentS + '_' + methodN + '/'
FolderNames_file = pathBootIn + 'iter_folder_names.txt'

#path to structural connectivity matrices
pathDTI = pathMainDir + '/' + 'StrucCons' + '/'
#path to fmri timeseries
pathfMRI = pathMainDir + '/' + 'timeseries' + '/'
pathfMRIC = pathMainDir + '/' + 'timeseries_clean' + '/'
#path to library cs_amd library
myCSparceLibP = "/vol/vipdata/users/fd301/packages/CSparse/Lib/libctest.so.1.0"

#bootstrap-----
#create directory if missing
if not os.path.exists(pathBootOut):
  os.makedirs(pathBootOut)

BootstrapIters = range(4750,5000)

if(methodN == 'symamd' ):
  pathOrderF = pathBootIn + 'symamdOrder' + '/' + 'OrderM_' + perCentS + '.txt'
  f = open(pathOrderF, 'r')
  content = f.readlines()
  f.close()
  OrderMTot = map(str.split,content)
  OrderMTot = np.float64(OrderMTot).astype(int)
  
#create sample
if not os.path.exists(FolderNames_file):
  folder_namesTOT = list()
  prob_allTOT = list()
  f = open(FolderNames_file, 'w')
  for biter in BootstrapIters:
    folder_names_tmp = list()
    for ii in range(0,subjNum):
      folder_names_tmp.append(folder_names00[ rand.randint(0,subjNum-1) ] )
    f.write("%s\n" % ' '.join(folder_names_tmp))
    folder_namesTOT.append(folder_names_tmp)	
  f.close()
else:
  f = open(FolderNames_file, 'r')
  content = f.readlines()
  f.close()
  folder_namesTOT = map(str.split,content)
   
#bootstrap-----
prob_allTOT = list()
for biter in BootstrapIters:
    print(biter)
    
    folder_names = folder_namesTOT[biter]
    print(folder_names)	

    file_out = pathBootOut + "probAll_" + str(biter) + ".txt" 
    file_out_ri0 = pathBootOut + "ri0_" + str(biter) + ".txt"
    file_out_ci0 = pathBootOut + "ci0_" + str(biter) + ".txt"
    file_out_thres = pathBootOut + "thres_" + str(biter) + ".txt"
    
    if os.path.exists(file_out):
      print "skip:"+ str(biter)
      continue

    #load structural connectivity matrices
    print "Loading structural data ..."
    scAll = np.empty([lenR,lenR,subjNum])
    for ii,subj in enumerate(folder_names):
      inputP =  pathDTI + 'con_' + subj + '.txt'
      sc = np.loadtxt(inputP)
      sc = sc[rois,]
      sc = sc[:,rois]
      sc =np.maximum(sc,sc.T)  		#connectivity matrix need to be symmetrised: here I use the max
      scAll[:,:,ii] = sc 

    #run the script that estimates structural support 
    #given the percentage of desirable connections included
    print "Estimasting structural support ..."
    sp,thres = StrucSup(perCent,scAll)
    
    #provide the ordering that would result in the sparser cholesky
    order = 1		#0:natural ordering, 1:Chol, 2:LU, 3:QR
    if( methodN=='rand' ):
      print "Estimasting a random permutation ..."     
      spOrder = np.random.permutation(lenR)
    elif(methodN == 'symamd'):
      spOrder = OrderMTot[biter,]
    else:
      print "Estimasting AMD ordering ..."
      spOrder = cs_amdW(sp, order, myCSparceLibP)
    
    #estimate precision matrix from either Gaussian graphical model or
    #any correlation matrix (here we are using the second)
    #note that with the typical correlation coefficients the estimation
    #of the covariance matrix is not well regularised especially with a large number of rois.
    #This problem is exacerbated with the inversion of the correlation matrix 
    print "Loading fMRI averaged signal ..."
    precAll = np.empty([lenR,lenR,subjNum])
    for ii,subj in enumerate(folder_names):
      outputP = pathfMRIC + 'timeSeriesR' + prefS + '_' + subj + '.txt'
      inputP = pathfMRI + 'timeSeriesR' + prefS + '_' + subj + '.mat'
      data = io.loadmat(inputP)
      signal = data['MM'+prefS].T[:-8]
      dim = len(signal)
      conf = data['MM'+prefS].T[dim:dim+8]
      #average of white matter and csf as well as the motion parameters parameters 
      #must be regressed from the signal
      signal = clean_signals(signal,low_pass=False,confounds=conf)
      cor = np.corrcoef(signal)
      prec = prop_scaling(cor, sp)  #the gaussian graphical model --not provided
      #LedW = LedoitWolf(store_precision=True, assume_centered=False)	
      #LedW.fit(signal.T)
      #prec = LedW.get_precision()   #estimate the precision matrix based on the LedoitWolf regularisation
      precAll[:,:,ii] = prec


    print "Preparing Data for randomised Lasso: Cholesky decomposition, scaling diagonal and so on..."    
    #prepare data for ransomised Lasso
    Y,X,ri0,ci0 = prepChol(scAll,precAll,sp,spOrder)

    print "Randomised Lasso is running ..."
    try:
      prob_all = run_randomisedLasso(X.T,Y)
    except(RuntimeError):
      print(RuntimeError)
      continue
    except(TypeError):
      print(TypeError)
      continue
    except:
      print(triplet)
      print "Unexpected error:", sys.exc_info()[0]
      continue 

    if(save_flag):
      prob_all = np.array(prob_all)     
      np.savetxt(file_out,np.array(prob_all))
      np.savetxt(file_out_ri0,ri0)
      np.savetxt(file_out_ci0,ci0)
      
    #print "Reorder data and visualise ..."
    #adj2,adj = visual_res(prob_all,lenR,roisF,ri0,ci0,thres)
    #pl.matshow(adj2)
    #pl.show()
	

print "Good bye!"

